<?php

namespace FiloBlu\Flow\Console\Command;

use Exception;
use FiloBlu\Flow\Model\Inboundflow;
use FiloBlu\Flow\Model\ResourceModel\Inboundflow\CollectionFactory;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\ObjectManagerInterface;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class ResetCommand
 * @package FiloBlu\Flow\Console\Command
 */
class ResetCommand extends Command
{
    /**
     * @var CollectionFactory
     */
    protected $inboundflowFactory;

    /**
     * @var ObjectManagerInterface
     */
    protected $objectManager;
    /**
     * @var ResourceConnection
     */
    protected $resourceConnection;

    /**
     * ListCommand constructor.
     * @param CollectionFactory $inboundflowFactory
     * @param ResourceConnection $resourceConnection
     * @param ObjectManagerInterface $objectManager
     * @param string $name
     */
    public function __construct(
        CollectionFactory $inboundflowFactory,
        ResourceConnection $resourceConnection,
        ObjectManagerInterface $objectManager,
        $name = null
    ) {
        $this->inboundflowFactory = $inboundflowFactory;
        $this->objectManager = $objectManager;
        $this->resourceConnection = $resourceConnection;
        parent::__construct($name);
    }

    /**
     *
     */
    protected function configure()
    {
        $this->setName('flow:reset')
            ->addArgument('status', InputArgument::REQUIRED, 'Status to set')
            ->addArgument('flow', InputArgument::REQUIRED | InputArgument::IS_ARRAY, 'Inbound Flow ids')
            ->setDescription('Reset to state');
    }

    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     * @throws Exception
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $status = $input->getArgument('status');

        if (!InboundFlow::isValidStatus($status)) {
            $output->writeln("Invalid status '{$status}'");
            foreach (array_keys(InboundFlow::getStatusList()) as $allowed) {
                $output->writeln("    {$allowed}");
            }

            exit(1);
        }

        $items = $this->inboundflowFactory->create()
            ->addFilter('id', $input->getArgument('flow'), 'in')
            ->getItems();

        foreach ($items as $item) {
            $this->resetItemToState($item, $status);
        }
    }

    /**
     * @param Inboundflow $item
     * @param $status
     * @throws Exception
     */
    protected function resetItemToState(Inboundflow $item, $status)
    {
        $mainTable = $this->objectManager
            ->create(sprintf('FiloBlu\Flow\Model\%s', $item->getFlow()))
            ->getResource()
            ->getMainTable();

        $connection = $this->resourceConnection->getConnection();
        $where = $connection->quoteInto('meta_file = ?', $item->getId());

        switch ($status) {
            case Inboundflow::STATUS_PARSED:
                $bind = [
                    'meta_processed'     => 0,
                    'image_process_time' => null
                ];
                if ($item->getFlow() === 'Imagesflow') {
                    $bind['image_status'] = $status;
                }

                $connection->update($mainTable, $bind, $where);
                break;
            case Inboundflow::STATUS_RECEIVED:
                $connection->delete($mainTable, $where);
                break;
            default:
        }

        $item->setStatus($status);
        $item->setRetry(0);
        $item->setLog('');
        $item->setComment('');
        $item->save();
    }
}
